@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: checkpw.c,v 1.72 2004/03/17 13:58:13 rjs3 Exp $
+ * $Id: checkpw.c,v 1.79 2009/05/08 00:43:44 murch Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -74,6 +74,7 @@
 #include <string.h>
 #endif
 
+#include <limits.h>
 #include <sys/types.h>
 #include <ctype.h>
 
@@ -117,7 +118,7 @@ static int _sasl_make_plain_secret(const char *salt,
     _sasl_MD5Init(&ctx);
     _sasl_MD5Update(&ctx, salt, 16);
     _sasl_MD5Update(&ctx, "sasldb", 6);
-    _sasl_MD5Update(&ctx, passwd, passlen);
+    _sasl_MD5Update(&ctx, passwd, (unsigned int) passlen);
     memcpy((*secret)->data, salt, 16);
     (*secret)->data[16] = '\0';
     _sasl_MD5Final((*secret)->data + 17, &ctx);
@@ -126,7 +127,7 @@ static int _sasl_make_plain_secret(const char *salt,
     return SASL_OK;
 }
 
-/* erase & dispose of a sasl_secret_t
+/* verify user password using auxprop plugins
  */
 static int auxprop_verify_password(sasl_conn_t *conn,
 				   const char *userstr,
@@ -135,8 +136,6 @@ static int auxprop_verify_password(sasl_conn_t *conn,
 				   const char *user_realm __attribute__((unused)))
 {
     int ret = SASL_FAIL;
-    char *userid = NULL;
-    char *realm = NULL;
     int result = SASL_OK;
     sasl_server_conn_t *sconn = (sasl_server_conn_t *)conn;
     const char *password_request[] = { SASL_AUX_PASSWORD,
@@ -150,28 +149,32 @@ static int auxprop_verify_password(sasl_conn_t *conn,
     /* We need to clear any previous results and re-canonify to 
      * ensure correctness */
 
-    prop_clear(sconn->sparams->propctx, 0);
+    prop_clear (sconn->sparams->propctx, 0);
 	
     /* ensure its requested */
     result = prop_request(sconn->sparams->propctx, password_request);
 
     if(result != SASL_OK) return result;
 
-    result = _sasl_canon_user(conn, userstr, 0,
-			      SASL_CU_AUTHID | SASL_CU_AUTHZID,
-			      &(conn->oparams));
+    result = _sasl_canon_user_lookup (conn,
+				      userstr,
+				      0,
+				      SASL_CU_AUTHID | SASL_CU_AUTHZID,
+				      &(conn->oparams));
     if(result != SASL_OK) return result;
     
     result = prop_getnames(sconn->sparams->propctx, password_request,
 			   auxprop_values);
-    if(result < 0)
+    if (result < 0) {
 	return result;
+    }
 
-    if((!auxprop_values[0].name
-         || !auxprop_values[0].values || !auxprop_values[0].values[0])
-       && (!auxprop_values[1].name
-         || !auxprop_values[1].values || !auxprop_values[1].values[0]))
-	    return SASL_NOUSER;
+    /* Verify that the returned <name>s are correct.
+       But we defer checking for NULL values till after we verify
+       that a passwd is specified. */
+    if (!auxprop_values[0].name && !auxprop_values[1].name) {
+	return SASL_NOUSER;
+    }
         
     /* It is possible for us to get useful information out of just
      * the lookup, so we won't check that we have a password until now */
@@ -180,6 +183,11 @@ static int auxprop_verify_password(sasl_conn_t *conn,
 	goto done;
     }
 
+    if ((!auxprop_values[0].values || !auxprop_values[0].values[0])
+	&& (!auxprop_values[1].values || !auxprop_values[1].values[0])) {
+	return SASL_NOUSER;
+    }
+        
     /* At the point this has been called, the username has been canonified
      * and we've done the auxprop lookup.  This should be easy. */
     if(auxprop_values[0].name
@@ -220,9 +228,102 @@ static int auxprop_verify_password(sasl_conn_t *conn,
 				      password_request[0]);
 
  done:
-    if (userid) sasl_FREE(userid);
-    if (realm)  sasl_FREE(realm);
+    /* We're not going to erase the property here because other people
+     * may want it */
+    return ret;
+}
 
+/* Verify user password using auxprop plugins. Allow verification against a hashed password,
+ * or non-retrievable password. Don't use cmusaslsecretPLAIN attribute.
+ *
+ * This function is similar to auxprop_verify_password().
+ */
+static int auxprop_verify_password_hashed(sasl_conn_t *conn,
+					  const char *userstr,
+					  const char *passwd,
+					  const char *service __attribute__((unused)),
+					  const char *user_realm __attribute__((unused)))
+{
+    int ret = SASL_FAIL;
+    int result = SASL_OK;
+    sasl_server_conn_t *sconn = (sasl_server_conn_t *)conn;
+    const char *password_request[] = { SASL_AUX_PASSWORD,
+				       NULL };
+    struct propval auxprop_values[2];
+    unsigned extra_cu_flags = 0;
+
+    if (!conn || !userstr)
+	return SASL_BADPARAM;
+
+    /* We need to clear any previous results and re-canonify to 
+     * ensure correctness */
+
+    prop_clear(sconn->sparams->propctx, 0);
+	
+    /* ensure its requested */
+    result = prop_request(sconn->sparams->propctx, password_request);
+
+    if (result != SASL_OK) return result;
+
+    /* We need to pass "password" down to the auxprop_lookup */
+    /* NB: We don't support binary passwords */
+    if (passwd != NULL) {
+	prop_set (sconn->sparams->propctx,
+		  SASL_AUX_PASSWORD,
+		  passwd,
+		  -1);
+	extra_cu_flags = SASL_CU_VERIFY_AGAINST_HASH;
+    }
+
+    result = _sasl_canon_user_lookup (conn,
+				      userstr,
+				      0,
+				      SASL_CU_AUTHID | SASL_CU_AUTHZID | extra_cu_flags,
+				      &(conn->oparams));
+
+    if (result != SASL_OK) return result;
+    
+    result = prop_getnames(sconn->sparams->propctx, password_request,
+			   auxprop_values);
+    if (result < 0) {
+	return result;
+    }
+
+    /* Verify that the returned <name>s are correct.
+       But we defer checking for NULL values till after we verify
+       that a passwd is specified. */
+    if (!auxprop_values[0].name && !auxprop_values[1].name) {
+	return SASL_NOUSER;
+    }
+        
+    /* It is possible for us to get useful information out of just
+     * the lookup, so we won't check that we have a password until now */
+    if (!passwd) {
+	ret = SASL_BADPARAM;
+	goto done;
+    }
+
+    if ((!auxprop_values[0].values || !auxprop_values[0].values[0])) {
+	return SASL_NOUSER;
+    }
+
+    /* At the point this has been called, the username has been canonified
+     * and we've done the auxprop lookup.  This should be easy. */
+
+    /* NB: Note that if auxprop_lookup failed to verify the password,
+       then the userPassword property value would be NULL */
+    if (auxprop_values[0].name
+        && auxprop_values[0].values
+        && auxprop_values[0].values[0]
+        && !strcmp(auxprop_values[0].values[0], passwd)) {
+	/* We have a plaintext version and it matched! */
+	return SASL_OK;
+    } else {
+	/* passwords do not match */
+	ret = SASL_BADAUTH;
+    }
+
+ done:
     /* We're not going to erase the property here because other people
      * may want it */
     return ret;
@@ -590,18 +691,32 @@ static int saslauthd_verify_password(sasl_conn_t *conn,
      * count authid count password count service count realm
      */
     {
- 	unsigned short u_len, p_len, s_len, r_len;
+ 	unsigned short max_len, req_len, u_len, p_len, s_len, r_len;
  
+	max_len = (unsigned short) sizeof(query);
+
+	/* prevent buffer overflow */
+	if ((strlen(userid) > USHRT_MAX) ||
+	    (strlen(passwd) > USHRT_MAX) ||
+	    (strlen(service) > USHRT_MAX) ||
+	    (user_realm && (strlen(user_realm) > USHRT_MAX))) {
+	    goto toobig;
+	}
+
  	u_len = (strlen(userid));
  	p_len = (strlen(passwd));
 	s_len = (strlen(service));
 	r_len = ((user_realm ? strlen(user_realm) : 0));
 
-	if (u_len + p_len + s_len + r_len + 30 > (unsigned short) sizeof(query)) {
-	    /* request just too damn big */
-            sasl_seterror(conn, 0, "saslauthd request too large");
-	    goto fail;
-	}
+	/* prevent buffer overflow */
+	req_len = 30;
+	if (max_len - req_len < u_len) goto toobig;
+	req_len += u_len;
+	if (max_len - req_len < p_len) goto toobig;
+	req_len += p_len;
+	if (max_len - req_len < s_len) goto toobig;
+	req_len += s_len;
+	if (max_len - req_len < r_len) goto toobig;
 
 	u_len = htons(u_len);
 	p_len = htons(p_len);
@@ -732,6 +847,10 @@ static int saslauthd_verify_password(sasl_conn_t *conn,
     sasl_seterror(conn, SASL_NOLOG, "authentication failed");
     return SASL_BADAUTH;
 
+ toobig:
+    /* request just too damn big */
+    sasl_seterror(conn, 0, "saslauthd request too large");
+
  fail:
     if (freeme) free(freeme);
     return SASL_FAIL;
@@ -961,6 +1080,7 @@ static int always_true(sasl_conn_t *conn,
 
 struct sasl_verify_password_s _sasl_verify_password[] = {
     { "auxprop", &auxprop_verify_password },
+    { "auxprop-hashed", &auxprop_verify_password_hashed },
 #ifdef HAVE_PWCHECK
     { "pwcheck", &pwcheck_verify_password },
 #endif
@@ -969,9 +1089,9 @@ struct sasl_verify_password_s _sasl_verify_password[] = {
 #endif
 #ifdef HAVE_AUTHDAEMON
     { "authdaemond", &authdaemon_verify_password },
-#endif     
+#endif
 #ifdef HAVE_ALWAYSTRUE
     { "alwaystrue", &always_true },
-#endif     
+#endif
     { NULL, NULL }
 };
