@@ -1,7 +1,7 @@
 /* CRAM-MD5 SASL plugin
  * Rob Siemborski
  * Tim Martin 
- * $Id: cram.c,v 1.83 2003/12/15 20:04:22 rjs3 Exp $
+ * $Id: cram.c,v 1.87 2011/09/07 13:19:44 murch Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -65,7 +65,7 @@
 
 /*****************************  Common Section  *****************************/
 
-static const char plugin_id[] = "$Id: cram.c,v 1.83 2003/12/15 20:04:22 rjs3 Exp $";
+static const char plugin_id[] = "$Id: cram.c,v 1.87 2011/09/07 13:19:44 murch Exp $";
 
 /* convert a string of 8bit chars to it's representation in hex
  * using lowercase letters
@@ -174,7 +174,7 @@ crammd5_server_mech_step1(server_context_t *text,
 	    
     /* we shouldn't have received anything */
     if (clientinlen != 0) {
-	SETERROR(sparams->utils, "CRAM-MD5 does not accpet inital data");
+	SETERROR(sparams->utils, "CRAM-MD5 does not accept inital data");
 	return SASL_BADPROT;
     }
     
@@ -198,7 +198,7 @@ crammd5_server_mech_step1(server_context_t *text,
 	     sparams->serverFQDN);
     
     *serverout = text->challenge;
-    *serveroutlen = strlen(text->challenge);
+    *serveroutlen = (unsigned) strlen(text->challenge);
     
     /* free stuff */
     sparams->utils->free(time);    
@@ -221,10 +221,12 @@ crammd5_server_mech_step2(server_context_t *text,
     char *userid = NULL;
     sasl_secret_t *sec = NULL;
     int pos;
-    unsigned len;
+    size_t len;
     int result = SASL_FAIL;
     const char *password_request[] = { SASL_AUX_PASSWORD,
+#if defined(OBSOLETE_CRAM_ATTR)
 				       "*cmusaslsecretCRAM-MD5",
+#endif
 				       NULL };
     struct propval auxprop_values[3];
     HMAC_MD5_CTX tmphmac;
@@ -265,8 +267,11 @@ crammd5_server_mech_step2(server_context_t *text,
 					   password_request,
 					   auxprop_values);
     if (result < 0 ||
-	((!auxprop_values[0].name || !auxprop_values[0].values) &&
-	 (!auxprop_values[1].name || !auxprop_values[1].values))) {
+	((!auxprop_values[0].name || !auxprop_values[0].values)
+#if defined(OBSOLETE_CRAM_ATTR)
+	  && (!auxprop_values[1].name || !auxprop_values[1].values)
+#endif
+	)) {
 	/* We didn't find this username */
 	sparams->utils->seterror(sparams->utils->conn,0,
 				 "no secret in database");
@@ -286,18 +291,20 @@ crammd5_server_mech_step2(server_context_t *text,
 	sec = sparams->utils->malloc(sizeof(sasl_secret_t) + len);
 	if (!sec) goto done;
 	
-	sec->len = len;
-	strncpy(sec->data, auxprop_values[0].values[0], len + 1);   
+	sec->len = (unsigned) len;
+	strncpy((char *)sec->data, auxprop_values[0].values[0], len + 1);   
 	
 	clear_md5state = 1;
 	/* Do precalculation on plaintext secret */
 	sparams->utils->hmac_md5_precalc(&md5state, /* OUT */
 					 sec->data,
 					 sec->len);
+#if defined(OBSOLETE_CRAM_ATTR)
     } else if (auxprop_values[1].name && auxprop_values[1].values) {
 	/* We have a precomputed secret */
 	memcpy(&md5state, auxprop_values[1].values[0],
 	       sizeof(HMAC_MD5_STATE));
+#endif
     } else {
 	sparams->utils->seterror(sparams->utils->conn, 0,
 				 "Have neither type of secret");
@@ -317,7 +324,7 @@ crammd5_server_mech_step2(server_context_t *text,
     sparams->utils->hmac_md5_import(&tmphmac, (HMAC_MD5_STATE *) &md5state);
     sparams->utils->MD5Update(&(tmphmac.ictx),
 			      (const unsigned char *) text->challenge,
-			      strlen(text->challenge));
+			      (unsigned) strlen(text->challenge));
     sparams->utils->hmac_md5_final((unsigned char *) &digest, &tmphmac);
     
     /* convert to base 16 with lower case letters */
@@ -370,7 +377,11 @@ static int crammd5_server_mech_step(void *conn_context,
     
     *serverout = NULL;
     *serveroutlen = 0;
-    
+
+    if (text == NULL) {
+	return SASL_BADPROT;
+    }
+
     /* this should be well more than is ever needed */
     if (clientinlen > 1024) {
 	SETERROR(sparams->utils, "CRAM-MD5 input longer than 1024 bytes");
@@ -507,13 +518,13 @@ static int crammd5_client_mech_step(void *conn_context,
 				    sasl_out_params_t *oparams)
 {
     client_context_t *text = (client_context_t *) conn_context;
-    const char *authid;
+    const char *authid = NULL;
     sasl_secret_t *password = NULL;
     unsigned int free_password = 0; /* set if we need to free password */
     int auth_result = SASL_OK;
     int pass_result = SASL_OK;
     int result;
-    int maxsize;
+    size_t maxsize;
     char *in16 = NULL;
 
     *clientout = NULL;
@@ -597,13 +608,13 @@ static int crammd5_client_mech_step(void *conn_context,
     
     maxsize = 32+1+strlen(oparams->authid)+30;
     result = _plug_buf_alloc(params->utils, &(text->out_buf),
-			     &(text->out_buf_len), maxsize);
+			     &(text->out_buf_len), (unsigned) maxsize);
     if (result != SASL_OK) goto cleanup;
     
     snprintf(text->out_buf, maxsize, "%s %s", oparams->authid, in16);
     
     *clientout = text->out_buf;
-    *clientoutlen = strlen(*clientout);
+    *clientoutlen = (unsigned) strlen(*clientout);
     
     /* set oparams */
     oparams->doneflag = 1;
