@@ -1,6 +1,6 @@
 /* Generic SASL plugin utility functions
  * Rob Siemborski
- * $Id: plugin_common.c,v 1.15 2003/09/14 13:38:28 ken3 Exp $
+ * $Id: plugin_common.c,v 1.22 2011/09/01 14:12:18 mel Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -45,7 +45,7 @@
 #include <config.h>
 #ifndef macintosh
 #ifdef WIN32
-# include <winsock.h>
+# include <winsock2.h>
 #else
 # include <sys/socket.h>
 # include <netinet/in.h>
@@ -152,7 +152,7 @@ int _plug_ipfromstring(const sasl_utils_t *utils, const char *addr,
 	return SASL_BADPARAM;
     }
 
-    len = ai->ai_addrlen;
+    len = (socklen_t) ai->ai_addrlen;
     memcpy(&ss, ai->ai_addr, len);
     freeaddrinfo(ai);
     sockaddr_unmapped((struct sockaddr *)&ss, &len);
@@ -230,7 +230,7 @@ int _plug_buf_alloc(const sasl_utils_t *utils, char **rwbuf,
 	}
 	*curlen = newlen;
     } else if(*rwbuf && *curlen < newlen) {
-	size_t needed = 2*(*curlen);
+	unsigned needed = 2*(*curlen);
 
 	while(needed < newlen)
 	    needed *= 2;
@@ -267,7 +267,7 @@ int _plug_strdup(const sasl_utils_t * utils, const char *in,
   strcpy((char *) *out, in);
 
   if (outlen)
-      *outlen = len;
+      *outlen = (int) len;
 
   return SASL_OK;
 }
@@ -280,7 +280,7 @@ void _plug_free_string(const sasl_utils_t *utils, char **str)
 
   len = strlen(*str);
 
-  utils->erasebuffer(*str, len);
+  utils->erasebuffer(*str, (unsigned int) len);
   utils->free(*str);
 
   *str=NULL;
@@ -290,7 +290,7 @@ void _plug_free_secret(const sasl_utils_t *utils, sasl_secret_t **secret)
 {
     if(!utils || !secret || !(*secret)) return;
 
-    utils->erasebuffer((*secret)->data, (*secret)->len);
+    utils->erasebuffer((char *)(*secret)->data, (*secret)->len);
     utils->free(*secret);
     *secret = NULL;
 }
@@ -343,7 +343,7 @@ int _plug_get_simple(const sasl_utils_t *utils, unsigned int id, int required,
     }
   
     /* Try to get the callback... */
-    ret = utils->getcallback(utils->conn, id, &simple_cb, &simple_context);
+    ret = utils->getcallback(utils->conn, id, (sasl_callback_ft *)&simple_cb, &simple_context);
 
     if (ret == SASL_FAIL && !required)
 	return SASL_OK;
@@ -405,7 +405,7 @@ int _plug_get_password(const sasl_utils_t *utils, sasl_secret_t **password,
 
     /* Try to get the callback... */
     ret = utils->getcallback(utils->conn, SASL_CB_PASS,
-			     &pass_cb, &pass_context);
+			     (sasl_callback_ft *)&pass_cb, &pass_context);
 
     if (ret == SASL_OK && pass_cb) {
 	ret = pass_cb(utils->conn, pass_context, SASL_CB_PASS, password);
@@ -451,7 +451,7 @@ int _plug_challenge_prompt(const sasl_utils_t *utils, unsigned int id,
 
     /* Try to get the callback... */
     ret = utils->getcallback(utils->conn, id,
-			     &chalprompt_cb, &chalprompt_context);
+			     (sasl_callback_ft *)&chalprompt_cb, &chalprompt_context);
 
     if (ret == SASL_OK && chalprompt_cb) {
 	ret = chalprompt_cb(chalprompt_context, id,
@@ -497,7 +497,7 @@ int _plug_get_realm(const sasl_utils_t *utils, const char **availrealms,
 
     /* Try to get the callback... */
     ret = utils->getcallback(utils->conn, SASL_CB_GETREALM,
-			     &realm_cb, &realm_context);
+			     (sasl_callback_ft *)&realm_cb, &realm_context);
 
     if (ret == SASL_OK && realm_cb) {
 	ret = realm_cb(realm_context, SASL_CB_GETREALM, availrealms, realm);
@@ -636,7 +636,6 @@ int _plug_decode(decode_context_t *text,
     int ret;
     
     *outputlen = 0;
-    *output = NULL;
 
     while (inputlen) { /* more input */
 	if (text->needsize) { /* need to get the rest of the 4-byte size */
@@ -668,6 +667,10 @@ int _plug_decode(decode_context_t *text,
 		if (text->buffer == NULL) return SASL_NOMEM;
 
 		text->cursize = 0;
+	    } else {
+		/* We do NOT have the entire 4-byte size...
+		 * wait for more data */
+		return SASL_OK;
 	    }
 	}
 
@@ -754,6 +757,29 @@ int _plug_parseuser(const sasl_utils_t *utils,
     return ret;
 }
 
+int _plug_make_fulluser(const sasl_utils_t *utils,
+			char **fulluser,
+			const char * useronly,
+			const char *realm)
+{
+    if(!fulluser || !useronly || !realm) {
+	PARAMERROR( utils );
+	return (SASL_BADPARAM);
+    }
+
+    *fulluser = utils->malloc (strlen(useronly) + strlen(realm) + 2);
+    if (*fulluser == NULL) {
+	MEMERROR( utils );
+	return (SASL_NOMEM);
+    }
+
+    strcpy (*fulluser, useronly);
+    strcat (*fulluser, "@");
+    strcat (*fulluser, realm);
+
+    return (SASL_OK);
+}
+
 char * _plug_get_error_message (const sasl_utils_t *utils,
 #ifdef WIN32
 				DWORD error
@@ -887,3 +913,12 @@ SKIP_OS_INFO:
     snprintf(osbuf, osbuf_len, "%s %s", os.sysname, os.release);
 #endif /* WIN32 */
 }
+
+#if defined(WIN32)
+unsigned int plug_sleep (unsigned int seconds)
+{
+    long dwSec = seconds*1000;
+    Sleep (dwSec);
+    return 0;
+}
+#endif
