@@ -55,6 +55,7 @@
 #include <openssl/des.h>
 #endif
 
+#define LDAP_DEPRECATED 1
 #include <ldap.h>
 #include <lber.h>
 #include <sasl.h>
@@ -159,8 +160,8 @@ static int lak_config_read(
 		if (buf[strlen(buf)-1] == '\n') 
 			buf[strlen(buf)-1] = '\0';
 		for (p = buf; *p && isspace((int) *p); p++);
-			if (!*p || *p == '#') 
-				continue;
+		if (!*p || *p == '#') 
+		    continue;
 
 		key = p;
 		while (*p && (isalnum((int) *p) || *p == '-' || *p == '_')) {
@@ -168,16 +169,20 @@ static int lak_config_read(
 				*p = tolower(*p);
 			p++;
 		}
-		if (*p != ':')
+		if (*p != ':') {
+			fclose(infile);
 			return LAK_FAIL;
+		}
 		
 		*p++ = '\0';
 
 		while (*p && isspace((int) *p)) 
 			p++;
 
-		if (!*p)
+		if (!*p) {
+			fclose(infile);
 			return LAK_FAIL;
+		}
 
 		if (!strcasecmp(key, "ldap_servers"))
 			strlcpy(conf->servers, p, LAK_URL_LEN);
@@ -501,11 +506,7 @@ static int lak_tokenize_domains(
 	s1 = (char *)strtok_r(s, ".", &lasts);
 	while(s1) {
 		if (i == 0) {
-			*result = strdup(s1);
-			if (*result != NULL)
-				rc = lak_escape(s1, strlen(s1), result);
-			else
-				rc = LAK_NOMEM;
+			rc = lak_escape(s1, strlen(s1), result);
 			free(s);
 			return rc;
 		}
@@ -1475,10 +1476,9 @@ static int lak_auth_fastbind(
 	if (lak->conf->use_sasl) {
 		strlcpy(id, user, LAK_BUF_LEN);
 		if (!strchr(id, '@') &&
-		    (ISSET(realm) ||
-		     ISSET(lak->conf->default_realm))) {
-				strlcat(id, "@", LAK_BUF_LEN);
-				strlcat(id, (ISSET(realm) ? realm : lak->conf->default_realm), LAK_BUF_LEN);
+		    (ISSET(realm))) {
+			strlcat(id, "@", LAK_BUF_LEN);
+			strlcat(id, realm, LAK_BUF_LEN);
 		}
 	} else {
 		rc = lak_expand_tokens(lak->conf->filter, user, service, realm, NULL, &dn);
